/* card.h
 */


#ifndef HAVE_CARD_H
#define HAVE_CARD_H


typedef enum {
    GROUP_NEW = 0,  /* Untested cards. */
    GROUP_1,        /* Least well known cards. */
    GROUP_2,
    GROUP_3,
    GROUP_4,
    GROUP_5,
    GROUP_6,
    GROUP_7,
    GROUP_8,        /* Best known cards. */
    N_GROUPS
} Group;

#define GROUP_MAX (N_GROUPS - 1)


typedef enum {
    DATE_CREATED = 0,
    DATE_TESTED,
    DATE_EXPIRY,
    N_CARD_DATES
} CardDate;


#define card_set_date_created(card, date)  card_set_date(card, DATE_CREATED, date)
#define card_get_gdate_created(card)  card_get_gdate(card, DATE_CREATED)
#define card_get_date_created(card)   card_get_date(card, DATE_CREATED)

#define card_set_date_tested(card, date)  card_set_date(card, DATE_TESTED, date)
#define card_get_gdate_tested(card)  card_get_gdate(card, DATE_TESTED)
#define card_get_date_tested(card)   card_get_date(card, DATE_TESTED)

#define card_set_date_expiry(card, date)  card_set_date(card, DATE_EXPIRY, date)
#define card_get_gdate_expiry(card)  card_get_gdate(card, DATE_EXPIRY)
#define card_get_date_expiry(card)   card_get_date(card, DATE_EXPIRY)

#define CARD(glist_item)     ((Card*)(glist_item)->data)
#define CATEGORY(glist_item) ((Category*)(glist_item)->data)

typedef struct _Card Card;
typedef struct _Category Category;


Card*            card_new();
void             card_free (Card *c);
void             card_list_free (GList *list, gboolean free_cards);

void             card_set_front (Card *c, const char *text);
void             card_set_back (Card *c, const char *text);
const gchar*     card_get_front (Card *c);
const gchar*     card_get_back (Card *c);

void             card_set_group (Card *c, Group g);
Group            card_get_group (Card *c);

void             card_set_category (Card *c, Category *cat);
Category*        card_get_category (Card *c);

void             card_set_date (Card *c, CardDate cd, guint32 date);
GDate*           card_get_gdate (Card *c, CardDate cd);
guint32          card_get_date (Card *c, CardDate cd);

void             card_set_n_tests (Card *c, guint n);
void             card_inc_n_tests (Card *c);
guint            card_get_n_tests (Card *c);

void             card_set_n_known (Card *c, guint n);
void             card_inc_n_known (Card *c);
guint            card_get_n_known (Card *c);

gfloat           card_get_score (Card *c);

void             card_set_expired (Card *c, gboolean expired);
gboolean         card_get_expired (Card *c);

void             card_reset_statistics (Card *c);

void             card_decrease_test_frequency (Card *c);

void             card_to_top (Card *c);

#if 0            /* XXX */
void             card_mark_tested (Prefs *p, Card *c, gboolean known);
#endif

Category*        category_new (const gchar *title);
void             category_free (Category *cat, gboolean free_cards);

void             category_set_title (Category *cat, const gchar *title);
const gchar*     category_get_title (Category *cat);

void             category_prepend_card (Category *cat, Card *c);
void             category_append_card (Category *cat, Card *c);
void             category_remove_card (Category *cat, Card *c);
GList*           category_get_cards (Category *cat);
guint            category_get_n_cards (Category *cat);


#endif  /* HAVE_CARD_H */

